@@ -3,12 +3,13 @@
 ##
 
 import re
-import sys
+from functools import reduce
+from operator import add
+
 import numpy as np
 import pandas as pd
-import scipy.sparse as sp
 
-from .meta import MetaFactor, MetaTerm, MetaFormula, MetaReal, MetaCateg
+from .meta import MetaFactor, MetaTerm, MetaFormula, MetaReal, MetaCateg, Drop, Column
 from .tools import (
     categorize, hstack, chainer, decorator, func_disp, valid_rows, split_size,
     atleast_2d, fillna, all_valid, splice, factorize_2d, onehot_encode
@@ -18,11 +19,14 @@ from .tools import (
 ## tools
 ##
 
-def is_categorical(ft):
-    if isinstance(ft, MetaFactor):
-        return isinstance(ft, MetaCateg)
-    elif isinstance(ft, MetaTerm):
-        return any(is_categorical(t) for t in ft)
+def is_categorical(x, strict=False):
+    agg = all if strict else any
+    if isinstance(x, MetaFactor):
+        return isinstance(x, MetaCateg)
+    elif isinstance(x, MetaTerm):
+        return agg([is_categorical(f) for f in x])
+    elif isinstance(x, MetaFormula):
+        return agg([is_categorical(t) for t in x])
 
 def ensure_tuple(t):
     if type(t) is tuple:
@@ -53,6 +57,10 @@ def robust_eval(data, expr, extern=None):
     else:
         return np.full(len(data), vals)
 
+# this works with __add__ overload
+def sum0(items):
+    return reduce(add, items)
+
 ##
 ## categoricals
 ##
@@ -86,63 +94,146 @@ def category_indices(vals, dropna=False, return_labels=False):
     else:
         return uni_ind1, valid
 
-# encode categories as one-hot matrix
-def encode_categorical(vals, names, method='sparse', drop=True):
+# splice out a category to -1 (in ordinal space)
+def zero_category(vals, labs, lab0):
+    try:
+        idx0 = labs.index(lab0)
+    except ValueError:
+        raise Exception(f'drop label not found: {lab0}')
+
+    # promote to zero index
+    labs = labs.copy()
+    labs = [labs.pop(idx0), *labs]
+
+    # shift value indices
+    vals = np.where(vals == idx0, -1, vals)
+    vals = np.where(vals < idx0, vals+1, vals)
+
+    return vals, labs
+
+# encode categories as one-hot matrix or ordinals
+def encode_categorical(vals, names, encoding='sparse', drop=Drop.FIRST):
     # reindex categoricals jointly
     cats_val, cats_lab, valid = category_indices(vals, return_labels=True)
     cats_lab = [swizzle(names, l) for l in cats_lab]
 
-    # if ordinal no labels are dropped
-    if method == 'ordinal':
-        cats_enc, cats_use = cats_val.reshape(-1, 1), cats_lab
-    elif method == 'sparse':
-        cats_enc, cats_all = onehot_encode(cats_val, drop=drop)
-        cats_use = [cats_lab[i] for i in cats_all]
+    # here 0 corresponds to dropped
+    dtype = drop_type(drop)
+    if dtype == Drop.VALUE:
+        sdrop = swizzle(names, drop)
+        cats_val, cats_lab = zero_category(cats_val, cats_lab, sdrop)
+        drop_first = True
+    else:
+        drop_first = dtype == Drop.FIRST
 
-    return cats_enc, cats_use, valid
+    # handle drop first case (now -1 is dropped)
+    if drop_first:
+        cats_val -= 1
+        cats_lab = cats_lab[1:]
+
+    # implement final encoding
+    if encoding == 'ordinal':
+        cats_enc = cats_val
+    elif encoding == 'sparse':
+        cats_enc = onehot_encode(cats_val)
+
+    return cats_enc, cats_lab, valid
 
 # subset data allowing for missing chunks
-def drop_invalid(valid, *mats, warn=True):
+def drop_invalid(valid, *mats, warn=False, name='data'):
     V, N = np.sum(valid), len(valid)
     if V == 0:
         raise Exception('all rows contain null data')
     elif V < N:
         if warn:
-            print(f'dropping {N-V}/{N} null data points')
+            print(f'dropping {N-V}/{N} null {name} rows')
         mats = [
             m[valid] if m is not None else None for m in mats
         ]
     return *mats,
 
-# remove unused categories (sparse `mat` requires `labels`)
-def prune_categories(mat, labels=None, method='sparse', warn=True):
-    # get positive count cats
-    if method == 'sparse':
-        vcats = np.ravel((mat!=0).sum(axis=0)) > 0
-        Kt = [len(ls) for ls in labels.values()]
-        tvalid = split_size(vcats, Kt)
-        P, K = np.sum(vcats), len(vcats)
-    elif method == 'ordinal':
-        vcats = [np.bincount(cm) > 0 for cm in mat.T]
-        P = sum([np.sum(vc) for vc in vcats])
-        K = sum([len(vc) for vc in vcats])
+def prune_sparse(labels, values, warn=True):
+    # get active categories
+    vcats = np.ravel((values!=0).sum(axis=0)) > 0
+    P = np.sum(vcats)
 
-    # prune if needed
-    if P < K:
-        if warn:
-            print(f'pruning {K-P}/{K} unused categories')
+    # get total categories
+    Kt = [len(ls) for ls in labels.values()]
+    K = sum(Kt)
 
-        # modify data matrix
-        if method == 'sparse':
-            mat = mat[:, vcats]
-            labels = {
-                t: [l for l, v in zip(ls, vs) if v]
-                for (t, ls), vs in zip(labels.items(), tvalid)
-            }
-        elif method == 'ordinal':
-            mat = np.vstack([category_indices(cm) for cm in mat.T]).T
+    # bail if total
+    if P == K:
+        return labels, values
+    if warn:
+        print(f'pruning {K-P}/{K} unused categories')
 
-    return mat, labels
+    # split back into terms
+    vcats = split_size(vcats, Kt)
+
+    # modify data matrices
+    values = values[:, vcats]
+    labels = {
+        t: [l for l, v in zip(ls, vs) if v]
+        for (t, ls), vs in zip(labels.items(), vcats)
+    }
+
+    return labels, values
+
+def prune_ordinal(labels, values, warn=True):
+    # get active categories
+    cvals, cinvs, cnums = zip(*[
+        np.unique(cm, return_inverse=True, return_counts=True) for cm in values.T
+    ])
+    P = sum([len(cn) for cn in cnums])
+
+    # get total categories
+    Kt = [len(ls) for ls in labels.values()]
+    K = sum(Kt)
+
+    # bail if total
+    if P >= K:
+        return labels, values
+    if warn:
+        print(f'pruning {K-P}/{K} unused categories')
+
+    # modify data matrices (tricky to exclude -1)
+    values = np.vstack([ci+np.min(cv) for cv, ci in zip(cvals, cinvs)]).T
+    labels = {
+        t: [ls[v] for v in cv if v != -1]
+        for (t, ls), cv in zip(labels.items(), cvals)
+    }
+
+    return labels, values
+
+# remove unused categories (sparse `values` requires `labels`)
+def prune_categories(labels, values, encoding='sparse', warn=True):
+    if encoding == 'sparse':
+        return prune_sparse(labels, values, warn=warn)
+    elif encoding == 'ordinal':
+        return prune_ordinal(labels, values, warn=warn)
+
+def drop_type(d):
+    return d if d in (Drop.FIRST, Drop.NONE) else Drop.VALUE
+
+def drop_repr(d):
+    t = drop_type(d)
+    if t == Drop.VALUE:
+        return ','.join([str(x) for x in d])
+    else:
+        _, s = str(t).split('.')
+        return s
+
+# aggregate drop values from list of Factors to Term
+def consensus_drop(drops):
+    if len(drops) == 0:
+        return None
+    udrop, *rdrop = set([drop_type(d) for d in drops])
+    if len(rdrop) > 0:
+        return Drop.FIRST
+    elif udrop == Drop.VALUE:
+        return tuple(drops)
+    else:
+        return udrop
 
 ##
 ## formula structure
@@ -176,7 +267,7 @@ class Factor(MetaFactor, metaclass=AccessorType):
         else:
             return False
 
-    def __repr__(self):
+    def __repr__(self, **kwargs):
         return self._name
 
     def __add__(self, other):
@@ -184,6 +275,8 @@ class Factor(MetaFactor, metaclass=AccessorType):
             return Formula(self, other)
         elif isinstance(other, MetaFormula):
             return Formula(self, *other)
+        else:
+            raise TypeError(f'Not a valid addition: {other}')
 
     def __sub__(self, other):
         return Formula(self) - other
@@ -195,23 +288,38 @@ class Factor(MetaFactor, metaclass=AccessorType):
             return Term(self, *other)
         elif isinstance(other, MetaFormula):
             return Formula(*[Term(self, *t) for t in other])
+        else:
+            raise TypeError(f'Not a valid multiplier: {other}')
 
     def __call__(self, *args, **kwargs):
         cls = type(self)
         return cls(self._expr, *args, **kwargs)
 
-    def to_term(self):
-        return Term(self)
+    def to_term(self, **kwargs):
+        return Term(self, **kwargs)
 
     def name(self):
         return self._name
 
-    def eval(self, data=None, extern=None):
+    def raw(self, data, extern=None):
         return robust_eval(data, self._expr, extern=extern)
 
+    def eval(self, data, **kwargs):
+        return self.to_term().eval(data, **kwargs)
+
 class Term(MetaTerm):
-    def __init__(self, *facts):
+    def __init__(self, *facts, drop=None):
         self._facts = facts
+        if drop is None:
+            if all(is_categorical(f) for f in facts):
+                self._drop = consensus_drop([f._drop for f in facts])
+            else:
+                self._drop = Drop.NONE
+        else:
+            if drop_type(drop) == Drop.VALUE:
+                self._drop = tuple(drop)
+            else:
+                self._drop = drop
 
     def __hash__(self):
         return hash(str(self))
@@ -228,7 +336,8 @@ class Term(MetaTerm):
         if len(self) == 0:
             return 'I'
         else:
-            return '*'.join([str(f) for f in self])
+            ds = f'|{drop_repr(self._drop)}' if self._drop != Drop.NONE else ''
+            return '*'.join([f.__repr__(drop=False) for f in self]) + ds
 
     def __iter__(self):
         return iter(self._facts)
@@ -241,6 +350,8 @@ class Term(MetaTerm):
             return Formula(self, other)
         elif isinstance(other, MetaFormula):
             return Formula(self, *other)
+        else:
+            raise TypeError(f'Not a valid addition: {other}')
 
     def __sub__(self, other):
         return Formula(self) - other
@@ -252,58 +363,96 @@ class Term(MetaTerm):
             return Term(*self, *other)
         elif isinstance(other, MetaFormula):
             return Formula(*[Term(*self, *t) for t in other])
+        else:
+            raise TypeError(f'Not a valid multiplier: {other}')
 
     def name(self):
         return '*'.join([f.name() for f in self])
 
+    def drop(self, *drop):
+        if len(drop) == 1:
+            drop0, = drop
+            if drop_type(drop0) == Drop.VALUE:
+                self._drop = drop
+            else:
+                self._drop = drop0
+        else:
+            self._drop = drop
+        return self
+
     def raw(self, data, extern=None):
-        return np.vstack([f.eval(data, extern=extern) for f in self]).T
+        return np.vstack([f.raw(data, extern=extern) for f in self]).T
+
+    def eval(self, data, extern=None, encoding='sparse'):
+        if encoding not in ('sparse', 'ordinal'):
+            raise ValueError(f'Unknown encoding: {encoding}')
 
-    def eval(self, data, method='sparse', drop=True, extern=None):
         # zero length is identity
         if len(self) == 0:
             N = len(data)
-            return np.ones((N, 1)), np.ones(N, dtype=bool), ['I']
+            return Column(
+                'I', 'I', np.ones(N), np.ones(N, dtype=bool)
+            )
 
         # separate pure real and categorical
         categ, reals = categorize(is_categorical, self)
         categ, reals = Term(*categ), Term(*reals)
 
+        # error with mixed types + encoding='ordinal'
+        if encoding == 'ordinal' and len(categ) > 0 and len(reals) > 0:
+            raise ValueError('Cannot use encoding="ordinal" with mixed factor types')
+
         # handle categorical
         if len(categ) > 0:
-            categ_mat = categ.raw(data, extern=extern)
+            categ_raw = categ.raw(data, extern=extern)
             categ_nam = [c.name() for c in categ]
             categ_value, categ_label, categ_valid = encode_categorical(
-                categ_mat, categ_nam, method=method, drop=drop
+                categ_raw, categ_nam, encoding=encoding, drop=self._drop
             )
 
         # handle reals
         if len(reals) > 0:
             reals_mat = reals.raw(data, extern=extern)
-            reals_value = reals_mat.prod(axis=1).reshape(-1, 1)
+            reals_value = reals_mat.prod(axis=1)
             reals_label = reals.name()
             reals_valid = valid_rows(reals_value)
 
         # combine results
+        name = self.name()
         if len(categ) == 0:
-            return reals_value, reals_valid, [reals_label]
+            return Column(name, reals_label, reals_value, reals_valid)
         elif len(reals) == 0:
-            return categ_value, categ_valid, categ_label
+            return Column(name, categ_label, categ_value, categ_valid)
         else:
             # filling nulls with 0 keeps sparse the same
-            term_value = categ_value.multiply(fillna(reals_value, v=0))
+            term_value = categ_value.multiply(fillna(reals_value, v=0)[:,None])
             term_label = [f'({l})*{reals_label}' for l in categ_label]
             term_valid = categ_valid & reals_valid
-            return term_value, term_valid, term_label
+            return Column(name, term_label, term_value, term_valid)
 
 class Formula(MetaFormula):
     def __init__(self, *terms):
+        # order preserving unique
         self._terms = tuple(dict.fromkeys(
             t if isinstance(t, MetaTerm) else Term(t) for t in terms
-        )) # order preserving unique
+        ))
+
+    def __eq__(self, other):
+        self_set = {frozenset(t) for t in self}
+        if isinstance(other, MetaFormula):
+            return self_set == {frozenset(t) for t in other}
+        if isinstance(other, MetaTerm):
+            return self_set == {frozenset(other)}
+        elif isinstance(other, MetaFactor):
+            return self_set == {frozenset({other})}
+        else:
+            return False
 
     def __repr__(self):
-        return ' + '.join(str(t) for t in self)
+        if len(self) == 0:
+            return 'O'
+        else:
+            return ' + '.join(str(t) for t in self)
 
     def __iter__(self):
         return iter(self._terms)
@@ -316,6 +465,8 @@ class Formula(MetaFormula):
             return Formula(*self, other)
         elif isinstance(other, MetaFormula):
             return Formula(*self, *other)
+        else:
+            raise TypeError(f'Not a valid addition: {other}')
 
     def __sub__(self, other):
         if isinstance(other, MetaFactor):
@@ -328,6 +479,8 @@ class Formula(MetaFormula):
             return Formula(*[
                 t for t in self if t not in other
             ])
+        else:
+            raise TypeError(f'Not a valid subtraction: {other}')
 
     def __mul__(self, other):
         if isinstance(other, MetaFactor):
@@ -338,59 +491,90 @@ class Formula(MetaFormula):
             return Formula(*chainer([
                 [Term(*t1, *t2) for t1 in self] for t2 in other
             ]))
+        else:
+            raise TypeError(f'Not a valid multiplier: {other}')
 
     def raw(self, data, extern=None):
         return [t.raw(data, extern=extern) for t in self]
 
-    def eval(self, data, method='sparse', drop=True, extern=None):
-        # split by all real or not
-        categ, reals = categorize(is_categorical, self)
+    def eval(self, data, extern=None, encoding='sparse', group=True, flatten=False):
+        if encoding not in ('sparse', 'ordinal'):
+            raise ValueError(f'Unknown encoding: {encoding}')
 
-        # handle categories
-        if len(categ) > 0:
-            categ_value, categ_valid, categ_label = zip(*[
-                t.eval(data, method=method, drop=drop, extern=extern)
-                for t in categ
-            ])
-            categ_value = hstack(categ_value)
-            categ_valid = np.vstack(categ_valid).all(axis=0)
+        # zero length is zero
+        if len(self) == 0:
+            N = len(data)
+            labels, values, valid = 'O', np.zeros(N), np.ones(N, dtype=bool)
+            if flatten:
+                return labels, values, valid
+            else:
+                return (labels, None), (values, None), valid
+
+        # get all term specs (type, labels, values, valid)
+        columns = [t.eval(data, extern=extern, encoding=encoding) for t in self]
+        valid = all_valid(*[c.valid for c in columns])
+
+        # just return raw specs
+        if not group and not flatten:
+            specs = [(c.name, c.labels, c.values) for c in columns]
+            return specs, valid
+
+        # group by real/categorical
+        if group or flatten:
+            # combine labels and default values
+            categ, reals = categorize(lambda c: type(c.labels) is list, columns)
+            reals_label = [c.labels for c in reals]
+            categ_label = {c.name: c.labels for c in categ}
+            reals_value, categ_value = None, None
+
+            # handle real terms
+            if len(reals) > 0:
+                reals_value = hstack([c.values[:,None] for c in reals])
+
+            # handle categorical terms
+            if len(categ) > 0:
+                if encoding == 'ordinal':
+                    categ_value = hstack([c.values[:,None] for c in categ])
+                elif encoding == 'sparse':
+                    categ_value = hstack([c.values for c in categ])
+
+        # do a full flatten?
+        if flatten:
+            labels = reals_label + chainer(categ_label.values())
+            values = hstack([reals_value, categ_value])
         else:
-            categ_value, categ_valid, categ_label = None, None, []
+            labels = reals_label, categ_label
+            values = reals_value, categ_value
 
-        # combine labels
-        categ_label = {t: ls for t, ls in zip(categ, categ_label)}
-
-        # handle reals
-        if len(reals) > 0:
-            reals_value, reals_valid, reals_label = zip(*[
-                t.eval(data, extern=extern) for t in reals
-            ])
-            reals_value = hstack(reals_value)
-            reals_label = chainer(reals_label)
-            reals_valid = np.vstack(reals_valid).all(axis=0)
-        else:
-            reals_value, reals_valid, reals_label = None, None, []
-
-        # return separately
-        return (
-            reals_value, reals_valid, reals_label,
-            categ_value, categ_valid, categ_label
-        )
+        return labels, values, valid
 
 ##
 ## column types
 ##
 
 class Real(MetaReal, Factor):
-    def __repr__(self):
+    def __repr__(self, **kwargs):
         return f'R({self.name()})'
 
 class Categ(MetaCateg, Factor):
-    def __repr__(self):
-        return f'C({self.name()})'
+    def __init__(self, expr, drop=Drop.FIRST, **kwargs):
+        super().__init__(expr, **kwargs)
+        self._drop = drop
+
+    def __repr__(self, drop=True, **kwargs):
+        nm = self.name()
+        if drop and self._drop != Drop.NONE:
+            ds = drop_repr(self._drop)
+            return f'C({nm}|{ds})'
+        else:
+            return f'C({nm})'
+
+    def drop(self, drop):
+        self._drop = drop
+        return self
 
 # custom columns — class interface
-# eval (mandatory): an ndarray of the values
+# raw (mandatory): an ndarray of the values
 # name (recommended): what gets displayed in the regression table
 # __repr__ (optional): what gets displayed on print [default to C/R(name)]
 
@@ -401,8 +585,8 @@ class Demean(Real):
         super().__init__(expr, name=f'{name}-μ{args}')
         self._cond = cond
 
-    def eval(self, data, extern=None):
-        vals = super().eval(data, extern=extern)
+    def raw(self, data, extern=None):
+        vals = super().raw(data, extern=extern)
         if self._cond is None:
             means = np.mean(vals)
         else:
@@ -414,18 +598,28 @@ class Demean(Real):
         return vals - means
 
 class Binned(Categ):
-    def __init__(self, expr, bins=10, labels=False, name=None):
+    def __init__(self, expr, bins=10, drop=Drop.FIRST, labels=False, name=None):
         nb = bins if type(bins) is int else len(bins)
         name = expr if name is None else name
-        super().__init__(expr, name=f'{name}:bin{nb}')
+        super().__init__(expr, drop=drop, name=f'{name}:bin{nb}')
         self._bins = bins
         self._labels = None if labels else False
 
-    def eval(self, data, extern=None):
-        vals = super().eval(data, extern=extern)
+    def raw(self, data, extern=None):
+        vals = super().raw(data, extern=extern)
         bins = pd.cut(vals, self._bins, labels=self._labels)
         return bins
 
+# shorthand for drop=NONE
+
+class Categ0(Categ):
+    def __init__(self, expr, *args, drop=None, **kwargs):
+        super().__init__(expr, *args, drop=Drop.NONE, **kwargs)
+
+class Binned0(Binned):
+    def __init__(self, expr, *args, drop=None, **kwargs):
+        super().__init__(expr, *args, drop=Drop.NONE, **kwargs)
+
 # custom columns — functional interface
 
 class Custom:
@@ -462,11 +656,14 @@ def factor(func, *args, **kwargs):
     return Custom(func, *args, **kwargs)
 
 # shortcuts
+O = Formula()
 I = Term()
 R = Real
 C = Categ
 D = Demean
 B = Binned
+C0 = Categ0
+B0 = Binned0
 
 ##
 ## conversion
@@ -491,11 +688,7 @@ def parse_term(term):
 
 # this can only handle treatment coding, but that's required for sparsity
 def parse_formula(form):
-    try:
-        from patsy.desc import ModelDesc
-    except:
-        print('Please install patsy for formula parsing')
-        return
+    from patsy.desc import ModelDesc
 
     # use patsy for formula parse
     desc = ModelDesc.from_formula(form)
@@ -545,46 +738,42 @@ def ensure_formula(y=None, x=None, formula=None):
     if formula is not None:
         y, x = parse_formula(formula)
     else:
-        y = parse_item(y) if y is not None else None
+        if isinstance(y, (MetaTerm, MetaFormula)):
+            raise Exception(f'LHS variable must be a single factor. Instead got: y = {y}')
+        else:
+            y = parse_item(y) if y is not None else None
         x = parse_list(x)
     return y, x
 
 def design_matrix(
-    y=None, x=None, formula=None, data=None, method='sparse', drop=True,
-    dropna=True, prune=True, warn=True, extern=None, valid0=None, flatten=True,
-    validate=False
+    x=None, formula=None, data=None, encoding='sparse', dropna=True, prune=True,
+    warn=True, extern=None, valid0=None, flatten=True, validate=False
 ):
-    y, x = ensure_formula(x=x, formula=formula)
-    if y is not None:
-        raise Exception('Use design_matrices for formulas with an LHS.')
+    _, x = ensure_formula(x=x, formula=formula)
 
     # evaluate x variables
-    x_mat, x_val, x_names, c_mat, c_val, c_labels = x.eval(
-        data, method=method, drop=drop, extern=extern
+    (x_lab, c_lab), (x_vec, c_vec), val = x.eval(
+        data, extern=extern, encoding=encoding, group=True
     )
 
     # aggregate valid info for data
-    valid = all_valid(valid0, x_val, c_val)
+    valid = all_valid(valid0, val)
 
     # drop null values if requested
     if dropna:
-        x_mat, c_mat = drop_invalid(
-            valid, x_mat, c_mat, warn=warn
-        )
+        x_vec, c_vec = drop_invalid(valid, x_vec, c_vec, warn=warn, name='x')
 
     # prune empty categories if requested
-    if prune and c_mat is not None:
-        c_mat, c_labels = prune_categories(
-            c_mat, c_labels, method=method, warn=warn
-        )
+    if prune and c_vec is not None:
+        c_lab, c_vec = prune_categories(c_lab, c_vec, encoding=encoding, warn=warn)
 
     # combine real and categorical?
     if flatten:
-        f_mat = hstack([x_mat, c_mat])
-        f_names = x_names + chainer(c_labels.values())
-        ret = f_mat, f_names
+        lab = x_lab + chainer(c_lab.values())
+        vec = hstack([x_vec, c_vec])
+        ret = lab, vec
     else:
-        ret = x_mat, x_names, c_mat, c_labels
+        ret = (x_lab, c_lab), (x_vec, c_vec)
 
     # return valid mask?
     if validate:
@@ -594,7 +783,7 @@ def design_matrix(
 
 def design_matrices(
     y=None, x=None, formula=None, data=None, dropna=True, extern=None,
-    valid0=None, validate=False, **kwargs
+    valid0=None, validate=False, warn=True, **kwargs
 ):
     # parse into pythonic formula system
     y, x = ensure_formula(x=x, y=y, formula=formula)
@@ -602,22 +791,23 @@ def design_matrices(
         raise Exception('Use design_matrix for formulas without an LHS')
 
     # get y data
-    y_vec, y_name = y.eval(data, extern=extern), y.name()
+    y_vec = y.raw(data, extern=extern)
+    y_lab = y.name()
     y_val = valid_rows(y_vec)
 
     # get valid x data
     x_val0 = all_valid(valid0, y_val)
-    *x_ret, valid = design_matrix(
-        x=x, data=data, dropna=dropna, extern=extern, valid0=x_val0,
+    x_lab, x_vec, valid = design_matrix(
+        x=x, data=data, dropna=dropna, extern=extern, valid0=x_val0, warn=False,
         validate=True, **kwargs
     )
 
     # drop invalid y
     if dropna:
-        y_vec, = drop_invalid(valid, y_vec)
+        y_vec, = drop_invalid(valid, y_vec, warn=warn, name='y')
 
     # return combined data
-    ret = y_vec, y_name, *x_ret
+    ret = y_lab, y_vec, x_lab, x_vec
     if validate:
         return *ret, valid
     else:
