@@ -3,15 +3,15 @@
 ##
 
 import re
-import numpy as np
-import pandas as pd
-import scipy.sparse as sp
 from functools import reduce
-from operator import mul, and_
+from operator import mul
+
+import numpy as np
+import scipy.sparse as sp
 
 from .tools import (
     hstack, multiply, ensure_dense, group_sums, group_means, chainer,
-    block_inverse, valid_rows
+    block_inverse, valid_rows, all_valid
 )
 from .formula import (
     category_indices, design_matrices, parse_tuple, parse_list, ensure_formula,
@@ -20,8 +20,8 @@ from .formula import (
 from .summary import param_table
 
 def ols(
-    y=None, x=None, formula=None, data=None, absorb=None, cluster=None,
-    hdfe=None, stderr=True, drop='first', extern=None, output='table'
+    y=None, x=None, formula=None, data=None, absorb=None, cluster=None, hdfe=None,
+    stderr=True, extern=None, warn=True, output='table'
 ):
     # convert to formula system
     y, x = ensure_formula(x=x, y=y, formula=formula)
@@ -32,13 +32,14 @@ def ols(
     # use hdfe trick
     if hdfe is not None:
         h_trm = parse_tuple(hdfe, convert=Categ)
+        h_nam = h_trm.name()
         x = (x - h_trm) + h_trm # pop to end
 
     # don't include absorb
     if absorb is not None:
         a_trm = parse_list(absorb, convert=Categ)
         a_mat = a_trm.raw(data, extern=extern)
-        valid &= reduce(and_, (valid_rows(a) for a in a_mat))
+        valid &= all_valid(*[valid_rows(a) for a in a_mat])
         x -= a_trm
         if cluster is None:
             cluster = reduce(mul, a_trm)
@@ -54,8 +55,8 @@ def ols(
         valid &= valid_rows(s_mat)
 
     # make design matrices
-    y_vec, y_name, x0_mat, x0_names, c_mat, c_names0, valid = design_matrices(
-        y=y, x=x, formula=formula, data=data, valid0=valid, drop=drop, extern=extern,
+    y_name, y_vec, (x0_names, c_names0), (x0_mat, c_mat), valid = design_matrices(
+        y=y, x=x, formula=formula, data=data, valid0=valid, extern=extern, warn=warn,
         flatten=False, validate=True
     )
 
@@ -117,7 +118,7 @@ def ols(
 
     # find inv(xpx) somehow
     if hdfe is not None:
-        Kh = len(c_names0[h_trm])
+        Kh = len(c_names0[h_nam])
         xh_mat, ch_mat = ensure_dense(x_mat[:, :-Kh]), x_mat[:, -Kh:]
         ixr, ixc = block_outer_inverse(xh_mat, ch_mat)
     else:
@@ -140,7 +141,10 @@ def ols(
     elif stderr is True:
         sigma = s2*ixpx
     elif type(stderr) is str:
-        hc, = map(int, re.match(r'hc([0-3])', stderr).groups())
+        hcret = re.match(r'hc([0-3])', stderr.lower())
+        if hcret is None:
+            raise Exception(f'Unrecognized covariance type {stderr}')
+        hc, = map(int, hcret.groups())
         sigma = hcn_stderr(hc, x_mat, xe_mat, ixpx)
 
     # return requested
