@@ -18,9 +18,17 @@ z95 = norm.ppf(0.975)
 ## param summary
 ##
 
-def param_table(beta, sigma, y_name, x_names):
-    # standard errors
-    if type(sigma) is tuple:
+def param_table(beta, y_name, x_names, sigma=None):
+    # basic frame
+    frame = pd.DataFrame({
+        'coeff': beta,
+    }, index=x_names)
+    frame = frame.rename_axis(y_name, axis=1)
+
+    # handle sigma cases
+    if sigma is None:
+        return frame
+    elif type(sigma) is tuple:
         sigr, sigc = sigma
         stderr = np.sqrt(np.hstack([maybe_diag(sigr), sigc]))
     else:
@@ -34,14 +42,9 @@ def param_table(beta, sigma, y_name, x_names):
     zscore = beta/stderr
     pvalue = 2*(1-norm.cdf(np.abs(zscore)))
 
-    # return all
-    frame = pd.DataFrame({
-        'coeff': beta,
-        'stderr': stderr,
-        'low95': low95,
-        'high95': high95,
-        'pvalue': pvalue
-    }, index=x_names)
-    frame = frame.rename_axis(y_name, axis=1)
+    # stderr stats
+    frame = frame.assign(
+        stderr=stderr, low95=low95, high95=high95, pvalue=pvalue
+    )
 
     return frame
