@@ -60,8 +60,8 @@ def dataset(
     if 'logit' in models:
         df['Eb0'] = 1/(1+np.exp(-df['yhat0']))
         df['Eb'] = 1/(1+np.exp(-df['yhat']))
-        df['b0'] = (st.randn(N) < df['Eb0']).astype(np.int)
-        df['b'] = (st.randn(N) < df['Eb']).astype(np.int)
+        df['b0'] = (st.rand(N) < df['Eb0']).astype(int)
+        df['b'] = (st.rand(N) < df['Eb']).astype(int)
 
     # poisson
     if 'poisson' in models:
@@ -70,8 +70,14 @@ def dataset(
         df['p0'] = st.poisson(df['Ep0'])
         df['p'] = st.poisson(df['Ep'])
 
+    # offset poisson
+    if 'poisson_offset' in models:
+        df['t'] = 1 + st.exponential(size=N)
+        df['pt0'] = st.poisson(df['Ep0']*df['t'])
+        df['pt'] = st.poisson(df['Ep']*df['t'])
+
     # zero-inflated poisson
-    if 'zinf_poisson' in models:
+    if 'poisson_zinf' in models:
         df['pz0'] = np.where(st.rand(N) < params['pz'], 0, df['p0'])
         df['pz'] = np.where(st.rand(N) < params['pz'], 0, df['p'])
 
@@ -81,30 +87,16 @@ def dataset(
         df['nb'] = rand_negbin(df['Ep'], params['alpha'], state=st)
 
     # zero-inflated poisson
-    if 'zinf_negbin' in models:
+    if 'negbin_zinf' in models:
         df['nbz0'] = np.where(st.rand(N) < params['pz'], 0, df['nb0'])
         df['nbz'] = np.where(st.rand(N) < params['pz'], 0, df['nb'])
 
+    # map id1 to letters
     if letter and K1 is not None:
         df['id1'] = df['id1'].map(lambda x: chr(65+x))
 
     return df
 
-def dataset_compare(N=10_000_000, K=100):
-    K1, K2 = N // K, K
-
-    id1 = np.random.randint(K1, size=N)
-    id2 = np.random.randint(K2, size=N)
-
-    x1 = 5*np.cos(id1) + 5*np.sin(id2) + np.random.randn(N)
-    x2 = np.cos(id1) + np.sin(id2) + np.random.randn(N)
-
-    y= 3*x1 + 5*x2 + np.cos(id1) + np.cos(id2)**2 + np.random.randn(N)
-
-    return pd.DataFrame({
-        'y': y, 'x1': x1, 'x2': x2, 'id1': id1,  'id2': id2
-    })
-
 def plot_coeff(beta, params=params0):
     import matplotlib.pyplot as plt
 
