@@ -2,12 +2,14 @@
 ## tools
 ##
 
+from operator import and_
+from itertools import chain, accumulate
+from functools import partial, reduce
+
 import numpy as np
 import pandas as pd
 import numpy.linalg as la
 import scipy.sparse as sp
-from itertools import chain, accumulate
-from functools import partial
 from pandas._libs.hashtable import SIZE_HINT_LIMIT
 from pandas.core.sorting import get_group_index
 
@@ -36,11 +38,13 @@ def maybe_diag(x):
     else:
         return x.diagonal()
 
-# handles empty data
+# allows None's and handles empty data
 def vstack(v, format='csr'):
     v = [x for x in v if x is not None]
     if len(v) == 0:
         return None
+    elif len(v) == 1:
+        return v[0]
     elif any([sp.issparse(x) for x in v]):
         return sp.vstack(v, format=format)
     else:
@@ -51,11 +55,25 @@ def hstack(v, format='csr'):
     v = [x for x in v if x is not None]
     if len(v) == 0:
         return None
+    elif len(v) == 1:
+        return v[0]
     elif any([sp.issparse(x) for x in v]):
         return sp.hstack(v, format=format)
     else:
         return np.hstack(v)
 
+# allows None's and handles empty data
+def vstack(v, format='csr'):
+    v = [x for x in v if x is not None]
+    if len(v) == 0:
+        return None
+    elif len(v) == 1:
+        return v[0]
+    elif any([sp.issparse(x) for x in v]):
+        return sp.vstack(v, format=format)
+    else:
+        return np.vstack(v)
+
 def get_dtype(*v):
     tarr = [x.dtype for x in v if hasattr(x, 'dtype')]
     tsca = [type(x) for x in v if not hasattr(x, 'dtype')]
@@ -158,8 +176,12 @@ def valid_rows(x):
         print(f'valid_rows: unsupported format "{fmt}"')
     return ~null
 
+# aggregate valid masks
 def all_valid(*mats):
-    return np.vstack([m for m in mats if m is not None]).all(axis=0)
+    mats = [x for x in mats if x is not None]
+    if len(mats) == 0:
+        return None
+    return reduce(and_, mats)
 
 # list based cumsum
 def cumsum(x):
@@ -173,17 +195,14 @@ def split_size(x, s):
 # expects compact positive integers, except for -1 being null
 # drops -1 always and 0 if drop (maps into all zeros)
 def onehot_encode(indx, drop=True):
-    base = 1 if drop else 0
-    sel = indx >= base
-    rows = np.flatnonzero(sel)
-    cols = indx[sel] - base
+    sel = indx >= 0
+    rows, cols = np.flatnonzero(sel), indx[sel]
     data = np.ones_like(rows)
-    N, K = len(indx), indx.max()
+    N, K = len(indx), cols.max() + 1
     smat = sp.csr_matrix(
         (data, (rows, cols)), shape=(N, K)
     )
-    cats = np.arange(base, K+1)
-    return smat, cats
+    return smat
 
 ##
 ## function tools
